@@ -4,19 +4,19 @@
 //! A flexible, simple to use, immutable, clone-efficient `String` replacement for Rust
 //!
 //! ```
-//! use flexstr::{flex_fmt, FlexStr, IntoFlexStr, ToCase, ToFlexStr};
+//! use flexstr::{flex_fmt, flex_str, FlexStr, IntoFlexStr, ToCase, ToFlexStr};
 //!
-//! // Use an `into` function to wrap a literal, no allocation or copying
-//! let static_str = "This will not allocate or copy".into_flex_str();
-//! assert!(static_str.is_static());
+//! // Use `flex_str` macro to wrap literals as compile-time constants
+//! const STATIC_STR: FlexStr = flex_str!("This will not allocate or copy");
+//! assert!(STATIC_STR.is_static());
 //!
 //! // Strings up to 22 bytes (on 64-bit) will be inlined automatically
-//! // (demo only, use `into` for literals as above)
+//! // (demo only, use macro or `from_static` for literals as above)
 //! let inline_str = "inlined".to_flex_str();
 //! assert!(inline_str.is_inlined());
 //!
 //! // When a string is too long to be wrapped/inlined, it will heap allocate
-//! // (demo only, use `into` for literals as above)
+//! // (demo only, use macro or `from_static` for literals as above)
 //! let rc_str = "This is too long to be inlined".to_flex_str();
 //! assert!(rc_str.is_heap());
 //!
@@ -37,32 +37,35 @@
 //! assert!(inline_str4.is_inlined());
 //! assert_eq!(inline_str4, "inlined!!!");
 //!
-//! // Clone is almost free, and never allocates
+//! // Clone is cheap, and never allocates
 //! // (at most it is a ref count increment for heap allocated strings)
-//! let static_str2 = static_str.clone();
-//! assert!(static_str2.is_static());
+//! let rc_str2 = rc_str.clone();
+//! assert!(rc_str2.is_heap());
 //!
 //! // Regardless of storage type, these all operate seamlessly together
 //! // and choose storage as required
-//! let heap_str2 = static_str2 + &inline_str;
+//! let heap_str2 = STATIC_STR + &inline_str;
 //! assert!(heap_str2.is_heap());
 //! assert_eq!(heap_str2, "This will not allocate or copyinlined");  
 //! ```
 
 extern crate alloc;
 
+#[doc(hidden)]
 #[macro_use]
-mod builder;
-mod inline;
-mod traits;
+pub mod builder;
+#[doc(hidden)]
+pub mod inline;
+#[doc(hidden)]
+pub mod traits;
 
 pub use inline::STRING_SIZED_INLINE;
+#[doc(inline)]
 pub use traits::*;
 
 use alloc::rc::Rc;
 use alloc::string::String;
 use alloc::sync::Arc;
-use core::borrow::Borrow;
 use core::cmp::Ordering;
 use core::convert::Infallible;
 use core::fmt;
@@ -90,42 +93,50 @@ mod test_readme {
         };
     }
 
-    external_doc_test!(include_str!("../README.md"));
+    external_doc_test!(include_str!("../../README.md"));
 }
 
+/// Error type returned from `try_to_static_str` when  this `FlexStr` does not contain a `'static str`
+#[derive(Copy, Clone, Debug)]
+pub struct NotStatic;
+
+#[doc(hidden)]
 #[derive(Clone, Debug)]
-enum FlexStrInner<const N: usize, T> {
-    /// A wrapped string literal
+pub enum FlexInner<const N: usize, T> {
+    // A wrapped string literal
     Static(&'static str),
-    /// An inlined string
+    // An inlined string
     Inlined(inline::InlineFlexStr<N>),
-    /// A reference count wrapped `str`
+    // A reference count wrapped `str`
     Heap(T),
 }
 
-/// A flexible string type that transparently wraps a string literal, inline string, or an `Rc<str>`
+/// A flexible string type that transparently wraps a string literal, inline string, or a heap allocated type
 #[derive(Clone, Debug)]
-pub struct FlexStr<const N: usize = STRING_SIZED_INLINE, T = Rc<str>>(FlexStrInner<N, T>);
+pub struct Flex<const N: usize, T>(#[doc(hidden)] pub FlexInner<N, T>);
+
+/// A flexible string type that transparently wraps a string literal, inline string, or an `Rc<str>`
+pub type FlexStr = Flex<STRING_SIZED_INLINE, Rc<str>>;
 
 /// A flexible string type that transparently wraps a string literal, inline string, or an `Arc<str>`
-pub type AFlexStr = FlexStr<STRING_SIZED_INLINE, Arc<str>>;
+pub type AFlexStr = Flex<STRING_SIZED_INLINE, Arc<str>>;
 
-impl<const N: usize, T> FlexStr<N, T> {
+impl<const N: usize, T> Flex<N, T> {
     /// Creates a wrapped static string literal. This function is equivalent to calling the `into`
     /// functions on a static string literal, but is `const fn` so can be used to init a constant.
     /// ```
     /// use flexstr::FlexStr;
     ///
-    /// const S: FlexStr = <FlexStr>::from_static("test");
+    /// const S: FlexStr = FlexStr::from_static("test");
     /// assert!(S.is_static());
     /// ```
     #[inline]
-    pub const fn from_static(s: &'static str) -> FlexStr<N, T> {
-        FlexStr(FlexStrInner::Static(s))
+    pub const fn from_static(s: &'static str) -> Flex<N, T> {
+        Flex(FlexInner::Static(s))
     }
 }
 
-impl<const N: usize, T> FlexStr<N, T>
+impl<const N: usize, T> Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -135,13 +146,13 @@ where
     /// ```
     /// use flexstr::FlexStr;
     ///
-    /// let s = <FlexStr>::try_inline("test").unwrap();
+    /// let s = FlexStr::try_inline("test").unwrap();
     /// assert!(s.is_inlined());
     /// ```
     #[inline]
-    pub fn try_inline(s: &str) -> Result<FlexStr<N, T>, &str> {
+    pub fn try_inline<S: AsRef<str>>(s: S) -> Result<Flex<N, T>, S> {
         match inline::InlineFlexStr::try_new(s) {
-            Ok(s) => Ok(FlexStr(FlexStrInner::Inlined(s))),
+            Ok(s) => Ok(Flex(FlexInner::Inlined(s))),
             Err(s) => Err(s),
         }
     }
@@ -152,15 +163,15 @@ where
     /// ```
     /// use flexstr::FlexStr;
     ///
-    /// let s = <FlexStr>::heap("test");
+    /// let s = FlexStr::heap("test");
     /// assert!(s.is_heap());
     /// ```
     #[inline]
-    pub fn heap(s: &str) -> FlexStr<N, T>
+    pub fn heap(s: impl AsRef<str>) -> Flex<N, T>
     where
         T: for<'a> From<&'a str>,
     {
-        FlexStr(FlexStrInner::Heap(s.into()))
+        Flex(FlexInner::Heap(s.as_ref().into()))
     }
 
     /// Returns the size of the maximum possible inline length for this type
@@ -180,9 +191,9 @@ where
     pub fn is_empty(&self) -> bool {
         // Due to how inline does deref, I'm guessing this is slightly cheaper by using inline native is_empty
         match &self.0 {
-            FlexStrInner::Static(str) => str.is_empty(),
-            FlexStrInner::Inlined(s) => s.is_empty(),
-            FlexStrInner::Heap(rc) => rc.is_empty(),
+            FlexInner::Static(str) => str.is_empty(),
+            FlexInner::Inlined(s) => s.is_empty(),
+            FlexInner::Heap(rc) => rc.is_empty(),
         }
     }
 
@@ -197,9 +208,9 @@ where
     pub fn len(&self) -> usize {
         // Due to how inline does deref, I'm guessing this is slightly cheaper by using inline native len
         match &self.0 {
-            FlexStrInner::Static(str) => str.len(),
-            FlexStrInner::Inlined(s) => s.len(),
-            FlexStrInner::Heap(rc) => rc.len(),
+            FlexInner::Static(str) => str.len(),
+            FlexInner::Inlined(s) => s.len(),
+            FlexInner::Heap(rc) => rc.len(),
         }
     }
 
@@ -213,80 +224,96 @@ where
     /// trait (which we cannot implement due to a blanket stdlib implementation) as this avoids the
     /// `Display`-based implementation.
     /// ```
-    /// use flexstr::IntoFlexStr;
+    /// use flexstr::flex_str;
     ///
-    /// let s = "abc".into_flex_str().to_string();
+    /// let s = flex_str!("abc").to_std_string();
     /// assert_eq!(s, "abc");
     /// ```
-    #[allow(clippy::inherent_to_string_shadow_display)]
     #[inline]
-    pub fn to_string(&self) -> String {
+    pub fn to_std_string(&self) -> String {
         String::from(&**self)
     }
 
+    /// Attempts to extract a static inline string literal if one is stored inside this `FlexStr`.
+    /// Returns `NotStatic` as an `Err` if this is not a static string literal.
+    /// ```
+    /// use flexstr::flex_str;
+    ///
+    /// let s = "abc";
+    /// let s2 = flex_str!(s);
+    /// assert_eq!(s2.try_to_static_str().unwrap(), s);
+    /// ```
+    #[inline]
+    pub fn try_to_static_str(&self) -> Result<&'static str, NotStatic> {
+        match self.0 {
+            FlexInner::Static(s) => Ok(s),
+            _ => Err(NotStatic),
+        }
+    }
+
     /// Returns true if this is a wrapped string literal (`&'static str`)
     /// ```
     /// use flexstr::FlexStr;
     ///
-    /// let s = <FlexStr>::from_static("test");
+    /// let s = FlexStr::from_static("test");
     /// assert!(s.is_static());
     /// ```
     #[inline]
     pub fn is_static(&self) -> bool {
-        matches!(self.0, FlexStrInner::Static(_))
+        matches!(self.0, FlexInner::Static(_))
     }
 
     /// Returns true if this is an inlined string
     /// ```
     /// use flexstr::FlexStr;
     ///
-    /// let s = <FlexStr>::try_inline("test").unwrap();
+    /// let s = FlexStr::try_inline("test").unwrap();
     /// assert!(s.is_inlined());
     /// ```
     #[inline]
     pub fn is_inlined(&self) -> bool {
-        matches!(self.0, FlexStrInner::Inlined(_))
+        matches!(self.0, FlexInner::Inlined(_))
     }
 
     /// Returns true if this is a wrapped string using heap storage
     /// ```
     /// use flexstr::FlexStr;
     ///
-    /// let s = <FlexStr>::heap("test");
+    /// let s = FlexStr::heap("test");
     /// assert!(s.is_heap());
     /// ```
     #[inline]
     pub fn is_heap(&self) -> bool {
-        matches!(self.0, FlexStrInner::Heap(_))
+        matches!(self.0, FlexInner::Heap(_))
     }
 }
 
 // *** Deref / Debug / Display ***
 
-impl<const N: usize, T> Deref for FlexStr<N, T>
+impl<const N: usize, T> Deref for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
     type Target = str;
 
     /// ```
-    /// use flexstr::IntoFlexStr;
+    /// use flexstr::flex_str;
     ///
     /// let a = "test";
-    /// let b = a.into_flex_str();
+    /// let b = flex_str!(a);
     /// assert_eq!(&*b, a);
     /// ```
     #[inline]
     fn deref(&self) -> &Self::Target {
         match &self.0 {
-            FlexStrInner::Static(str) => str,
-            FlexStrInner::Inlined(ss) => ss,
-            FlexStrInner::Heap(rc) => rc,
+            FlexInner::Static(str) => str,
+            FlexInner::Inlined(ss) => ss,
+            FlexInner::Heap(rc) => rc,
         }
     }
 }
 
-impl<const N: usize, T> Display for FlexStr<N, T>
+impl<const N: usize, T> Display for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -296,9 +323,38 @@ where
     }
 }
 
+#[cfg(feature = "fast_format")]
+impl<const N: usize, T> ufmt::uDisplay for Flex<N, T>
+where
+    T: Deref<Target = str>,
+{
+    #[inline]
+    fn fmt<W>(&self, f: &mut ufmt::Formatter<'_, W>) -> Result<(), W::Error>
+    where
+        W: ufmt_write::uWrite + ?Sized,
+    {
+        <str as ufmt::uDisplay>::fmt(self, f)
+    }
+}
+
+#[cfg(feature = "fast_format")]
+impl<const N: usize, T> ufmt::uDebug for Flex<N, T>
+where
+    T: Deref<Target = str>,
+{
+    #[inline]
+    fn fmt<W>(&self, f: &mut ufmt::Formatter<'_, W>) -> Result<(), W::Error>
+    where
+        W: ufmt_write::uWrite + ?Sized,
+    {
+        // uDebug is not implemented for str it seems which means we can't derive
+        <str as ufmt::uDisplay>::fmt(self, f)
+    }
+}
+
 // *** Hash, PartialEq, Eq ***
 
-impl<const N: usize, T> Hash for FlexStr<N, T>
+impl<const N: usize, T> Hash for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -308,7 +364,7 @@ where
     }
 }
 
-impl<const N: usize, T, T2> PartialEq<FlexStr<N, T2>> for FlexStr<N, T>
+impl<const N: usize, T, T2> PartialEq<Flex<N, T2>> for Flex<N, T>
 where
     T: Deref<Target = str>,
     T2: Deref<Target = str>,
@@ -322,12 +378,12 @@ where
     /// assert_eq!(s, s2);
     /// ```
     #[inline]
-    fn eq(&self, other: &FlexStr<N, T2>) -> bool {
+    fn eq(&self, other: &Flex<N, T2>) -> bool {
         str::eq(self, &**other)
     }
 }
 
-impl<const N: usize, T, T2> PartialEq<FlexStr<N, T2>> for &FlexStr<N, T>
+impl<const N: usize, T, T2> PartialEq<Flex<N, T2>> for &Flex<N, T>
 where
     T: Deref<Target = str>,
     T2: Deref<Target = str>,
@@ -341,12 +397,12 @@ where
     /// assert_eq!(&s, s2);
     /// ```
     #[inline]
-    fn eq(&self, other: &FlexStr<N, T2>) -> bool {
+    fn eq(&self, other: &Flex<N, T2>) -> bool {
         str::eq(self, &**other)
     }
 }
 
-impl<const N: usize, T> PartialEq<&str> for FlexStr<N, T>
+impl<const N: usize, T> PartialEq<&str> for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -363,7 +419,7 @@ where
     }
 }
 
-impl<const N: usize, T> PartialEq<str> for FlexStr<N, T>
+impl<const N: usize, T> PartialEq<str> for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -380,7 +436,7 @@ where
     }
 }
 
-impl<const N: usize, T> PartialEq<String> for FlexStr<N, T>
+impl<const N: usize, T> PartialEq<String> for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -397,11 +453,11 @@ where
     }
 }
 
-impl<const N: usize, T> Eq for FlexStr<N, T> where T: Deref<Target = str> {}
+impl<const N: usize, T> Eq for Flex<N, T> where T: Deref<Target = str> {}
 
 // *** PartialOrd / Ord ***
 
-impl<const N: usize, T> PartialOrd for FlexStr<N, T>
+impl<const N: usize, T> PartialOrd for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -411,7 +467,7 @@ where
     }
 }
 
-impl<const N: usize, T> PartialOrd<str> for FlexStr<N, T>
+impl<const N: usize, T> PartialOrd<str> for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -421,7 +477,7 @@ where
     }
 }
 
-impl<const N: usize, T> PartialOrd<String> for FlexStr<N, T>
+impl<const N: usize, T> PartialOrd<String> for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -431,7 +487,7 @@ where
     }
 }
 
-impl<const N: usize, T> Ord for FlexStr<N, T>
+impl<const N: usize, T> Ord for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -445,7 +501,7 @@ where
 
 macro_rules! impl_ranges {
     ($($type:ty),+) => {
-        $(impl<const N: usize, T> Index<$type> for FlexStr<N, T>
+        $(impl<const N: usize, T> Index<$type> for Flex<N, T>
         where
             T: Deref<Target = str>,
         {
@@ -471,9 +527,9 @@ impl_ranges!(
 // *** Add ***
 
 #[inline]
-fn concat<const N: usize, T>(s1: &str, s2: &str) -> FlexStr<N, T>
+fn concat<const N: usize, T>(s1: &str, s2: &str) -> Flex<N, T>
 where
-    T: From<String> + for<'a> From<&'a str>,
+    T: for<'a> From<&'a str>,
 {
     let mut buffer = buffer_new!(N);
     let mut builder = builder_new!(buffer, s1.len() + s2.len());
@@ -482,16 +538,16 @@ where
     builder_into!(builder, buffer)
 }
 
-impl<const N: usize, T> Add<&str> for FlexStr<N, T>
+impl<const N: usize, T> Add<&str> for Flex<N, T>
 where
-    T: From<String> + for<'a> From<&'a str> + Deref<Target = str>,
+    T: for<'a> From<&'a str> + Deref<Target = str>,
 {
-    type Output = FlexStr<N, T>;
+    type Output = Flex<N, T>;
 
     /// ```
-    /// use flexstr::IntoFlexStr;
+    /// use flexstr::{flex_str, IntoFlexStr};
     ///
-    /// let a = "in".into_flex_str() + "line";
+    /// let a = flex_str!("in") + "line";
     /// assert!(a.is_inlined());
     /// assert_eq!(a, "inline");
     ///
@@ -501,23 +557,29 @@ where
     /// ```
     #[inline]
     fn add(mut self, rhs: &str) -> Self::Output {
-        match self.0 {
-            FlexStrInner::Static(s) => concat(s, rhs),
-            FlexStrInner::Inlined(ref mut s) => {
-                if s.try_concat(rhs) {
-                    self
-                } else {
-                    concat(s, rhs)
+        if rhs.is_empty() {
+            self
+        } else if self.is_empty() {
+            rhs.into()
+        } else {
+            match self.0 {
+                FlexInner::Static(s) => concat(s, rhs),
+                FlexInner::Inlined(ref mut s) => {
+                    if s.try_concat(rhs) {
+                        self
+                    } else {
+                        concat(s, rhs)
+                    }
                 }
+                FlexInner::Heap(s) => concat(&s, rhs),
             }
-            FlexStrInner::Heap(s) => concat(&s, rhs),
         }
     }
 }
 
 // *** Misc. standard traits ***
 
-impl<const N: usize, T> AsRef<str> for FlexStr<N, T>
+impl<const N: usize, T> AsRef<str> for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -527,27 +589,14 @@ where
     }
 }
 
-impl<const N: usize, T> Default for FlexStr<N, T>
-where
-    T: Deref<Target = str>,
-{
+impl<const N: usize, T> Default for Flex<N, T> {
     #[inline]
     fn default() -> Self {
         Self::from_static("")
     }
 }
 
-impl<const N: usize, T> Borrow<str> for FlexStr<N, T>
-where
-    T: Deref<Target = str>,
-{
-    #[inline]
-    fn borrow(&self) -> &str {
-        str::borrow(self)
-    }
-}
-
-impl<const N: usize, T> FromStr for FlexStr<N, T>
+impl<const N: usize, T> FromStr for Flex<N, T>
 where
     T: for<'a> From<&'a str>,
 {
@@ -555,26 +604,26 @@ where
 
     #[inline]
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        Ok(s.to_flex())
+        Ok(s.into())
     }
 }
 
 // *** From ***
 
-impl<const N: usize, T, T2> From<&FlexStr<N, T2>> for FlexStr<N, T>
+impl<const N: usize, T, T2> From<&Flex<N, T2>> for Flex<N, T>
 where
-    T2: Clone,
-    FlexStr<N, T>: From<FlexStr<N, T2>>,
+    T: for<'a> From<&'a str>,
+    T2: Clone + Deref<Target = str>,
 {
     #[inline]
-    fn from(s: &FlexStr<N, T2>) -> Self {
-        s.clone().into()
+    fn from(s: &Flex<N, T2>) -> Self {
+        s.clone().into_flex()
     }
 }
 
-impl<const N: usize, T> From<String> for FlexStr<N, T>
+impl<const N: usize, T> From<String> for Flex<N, T>
 where
-    T: From<String>,
+    T: for<'a> From<&'a str>,
 {
     /// ```
     /// use flexstr::FlexStr;
@@ -591,14 +640,11 @@ where
     /// ```
     #[inline]
     fn from(s: String) -> Self {
-        FlexStr(match s.try_into() {
-            Ok(s) => FlexStrInner::Inlined(s),
-            Err(s) => FlexStrInner::Heap(s.into()),
-        })
+        <Self as From<&str>>::from(&s)
     }
 }
 
-impl<const N: usize, T> From<&String> for FlexStr<N, T>
+impl<const N: usize, T> From<&String> for Flex<N, T>
 where
     T: for<'a> From<&'a str>,
 {
@@ -617,31 +663,38 @@ where
     /// ```
     #[inline]
     fn from(s: &String) -> Self {
-        s.to_flex()
+        <Self as From<&str>>::from(s)
     }
 }
 
-impl<const N: usize, T> From<&'static str> for FlexStr<N, T>
+impl<const N: usize, T> From<&str> for Flex<N, T>
 where
-    T: Deref<Target = str>,
+    T: for<'a> From<&'a str>,
 {
     /// ```
     /// use flexstr::FlexStr;
     ///
-    /// let lit = "static";
+    /// let lit = "inline";
     /// let s: FlexStr  = lit.into();
-    /// assert!(s.is_static());
+    /// assert!(s.is_inlined());
     /// assert_eq!(&s, lit);
     /// ```
     #[inline]
-    fn from(s: &'static str) -> Self {
-        Self::from_static(s)
+    fn from(s: &str) -> Self {
+        if s.is_empty() {
+            Self::default()
+        } else {
+            Flex(match s.try_into() {
+                Ok(s) => FlexInner::Inlined(s),
+                Err(_) => FlexInner::Heap(s.into()),
+            })
+        }
     }
 }
 
-impl<const N: usize, T> From<char> for FlexStr<N, T>
+impl<const N: usize, T> From<char> for Flex<N, T>
 where
-    T: From<String> + for<'a> From<&'a str> + Deref<Target = str>,
+    T: Deref<Target = str>,
 {
     /// ```
     /// use flexstr::FlexStr;
@@ -660,10 +713,10 @@ where
 // *** FromIterator ***
 
 #[inline]
-fn from_iter_str<const N: usize, I, T, U>(iter: I) -> FlexStr<N, T>
+fn from_iter_str<const N: usize, I, T, U>(iter: I) -> Flex<N, T>
 where
     I: IntoIterator<Item = U>,
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
     U: AsRef<str>,
 {
     let iter = iter.into_iter();
@@ -673,17 +726,17 @@ where
     let mut buffer = buffer_new!(N);
     let mut builder = builder_new!(buffer);
     for s in iter {
-        builder.str_write(s.as_ref());
+        builder.str_write(s);
     }
     builder_into!(builder, buffer)
 }
 
 #[inline]
-fn from_iter_char<const N: usize, I, F, T, U>(iter: I, f: F) -> FlexStr<N, T>
+fn from_iter_char<const N: usize, I, F, T, U>(iter: I, f: F) -> Flex<N, T>
 where
     I: IntoIterator<Item = U>,
     F: Fn(U) -> char,
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
 {
     let iter = iter.into_iter();
     let (lower, _) = iter.size_hint();
@@ -696,13 +749,13 @@ where
     builder_into!(builder, buffer)
 }
 
-impl<const N: usize, T, T2> FromIterator<FlexStr<N, T2>> for FlexStr<N, T>
+impl<const N: usize, T, T2> FromIterator<Flex<N, T2>> for Flex<N, T>
 where
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
     T2: Deref<Target = str>,
 {
     /// ```
-    /// use flexstr::{FlexStr};
+    /// use flexstr::FlexStr;
     ///
     /// let v: Vec<FlexStr> = vec!["best".into(), "test".into()];
     /// let s: FlexStr = v.into_iter().map(|s| if s == "best" { "test".into() } else { s }).collect();
@@ -710,18 +763,18 @@ where
     /// assert_eq!(s, "testtest");
     /// ```
     #[inline]
-    fn from_iter<I: IntoIterator<Item = FlexStr<N, T2>>>(iter: I) -> Self {
+    fn from_iter<I: IntoIterator<Item = Flex<N, T2>>>(iter: I) -> Self {
         from_iter_str(iter)
     }
 }
 
-impl<'a, const N: usize, T, T2> FromIterator<&'a FlexStr<N, T2>> for FlexStr<N, T>
+impl<'a, const N: usize, T, T2> FromIterator<&'a Flex<N, T2>> for Flex<N, T>
 where
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
     T2: Deref<Target = str> + 'a,
 {
     /// ```
-    /// use flexstr::{FlexStr};
+    /// use flexstr::FlexStr;
     ///
     /// let v: Vec<FlexStr> = vec!["best".into(), "test".into()];
     /// let s: FlexStr = v.iter().filter(|s| *s == "best").collect();
@@ -729,17 +782,17 @@ where
     /// assert_eq!(s, "best");
     /// ```
     #[inline]
-    fn from_iter<I: IntoIterator<Item = &'a FlexStr<N, T2>>>(iter: I) -> Self {
+    fn from_iter<I: IntoIterator<Item = &'a Flex<N, T2>>>(iter: I) -> Self {
         from_iter_str(iter)
     }
 }
 
-impl<const N: usize, T> FromIterator<String> for FlexStr<N, T>
+impl<const N: usize, T> FromIterator<String> for Flex<N, T>
 where
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
 {
     /// ```
-    /// use flexstr::{FlexStr};
+    /// use flexstr::FlexStr;
     ///
     /// let v = vec!["best".to_string(), "test".to_string()];
     /// let s: FlexStr = v.into_iter().map(|s| if s == "best" { "test".into() } else { s }).collect();
@@ -752,12 +805,12 @@ where
     }
 }
 
-impl<'a, const N: usize, T> FromIterator<&'a str> for FlexStr<N, T>
+impl<'a, const N: usize, T> FromIterator<&'a str> for Flex<N, T>
 where
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
 {
     /// ```
-    /// use flexstr::{FlexStr};
+    /// use flexstr::FlexStr;
     ///
     /// let v = vec!["best", "test"];
     /// let s: FlexStr = v.into_iter().map(|s| if s == "best" { "test" } else { s }).collect();
@@ -770,12 +823,12 @@ where
     }
 }
 
-impl<const N: usize, T> FromIterator<char> for FlexStr<N, T>
+impl<const N: usize, T> FromIterator<char> for Flex<N, T>
 where
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
 {
     /// ```
-    /// use flexstr::{FlexStr};
+    /// use flexstr::FlexStr;
     ///
     /// let v = "besttest";
     /// let s: FlexStr = v.chars().map(|c| if c == 'b' { 't' } else { c }).collect();
@@ -788,12 +841,12 @@ where
     }
 }
 
-impl<'a, const N: usize, T> FromIterator<&'a char> for FlexStr<N, T>
+impl<'a, const N: usize, T> FromIterator<&'a char> for Flex<N, T>
 where
-    T: From<String> + for<'b> From<&'b str>,
+    T: for<'b> From<&'b str>,
 {
     /// ```
-    /// use flexstr::{FlexStr};
+    /// use flexstr::FlexStr;
     ///
     /// let v = vec!['b', 'e', 's', 't', 't', 'e', 's', 't'];
     /// let s: FlexStr = v.iter().filter(|&ch| *ch != 'b').collect();
@@ -809,7 +862,7 @@ where
 // *** Optional serialization support ***
 
 #[cfg(feature = "serde")]
-impl<const N: usize, T> Serialize for FlexStr<N, T>
+impl<const N: usize, T> Serialize for Flex<N, T>
 where
     T: Deref<Target = str>,
 {
@@ -829,9 +882,9 @@ struct FlexStrVisitor<const N: usize, T>(PhantomData<*const T>);
 #[cfg(feature = "serde")]
 impl<'de, const N: usize, T> Visitor<'de> for FlexStrVisitor<N, T>
 where
-    T: From<String> + for<'a> From<&'a str>,
+    T: for<'a> From<&'a str>,
 {
-    type Value = FlexStr<N, T>;
+    type Value = Flex<N, T>;
 
     #[inline]
     fn expecting(&self, formatter: &mut Formatter) -> fmt::Result {
@@ -843,7 +896,7 @@ where
     where
         E: Error,
     {
-        Ok(v.to_flex())
+        Ok(v.into())
     }
 
     #[inline]
@@ -856,9 +909,9 @@ where
 }
 
 #[cfg(feature = "serde")]
-impl<'de, const N: usize, T> Deserialize<'de> for FlexStr<N, T>
+impl<'de, const N: usize, T> Deserialize<'de> for Flex<N, T>
 where
-    T: From<String> + for<'a> From<&'a str>,
+    T: for<'a> From<&'a str>,
 {
     #[inline]
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
@@ -869,10 +922,40 @@ where
     }
 }
 
+/// Create compile time constant `FlexStr` (equivalent, but less typing than:
+/// `<FlexStr>::from_static("my_literal")`
+/// ```
+/// use flexstr::{flex_str, FlexStr};
+///
+/// const STR: FlexStr = flex_str!("This is a constant!");
+/// assert!(STR.is_static())
+/// ```
+#[macro_export]
+macro_rules! flex_str {
+    ($str:expr) => {
+        <$crate::FlexStr>::from_static($str)
+    };
+}
+
+/// Create compile time constant `AFlexStr` (equivalent, but less typing than:
+/// `<AFlexStr>::from_static("my_literal")`
+/// ```
+/// use flexstr::{a_flex_str, AFlexStr};
+///
+/// const STR: AFlexStr = a_flex_str!("This is a constant!");
+/// assert!(STR.is_static())
+/// ```
+#[macro_export]
+macro_rules! a_flex_str {
+    ($str:expr) => {
+        <$crate::AFlexStr>::from_static($str)
+    };
+}
+
 /// `FlexStr` equivalent to `format` function from stdlib. Efficiently creates a native `FlexStr`
-pub fn flex_fmt<const N: usize, T>(args: Arguments<'_>) -> FlexStr<N, T>
+pub fn flex_fmt<const N: usize, T>(args: Arguments<'_>) -> Flex<N, T>
 where
-    T: From<String> + for<'a> From<&'a str>,
+    T: for<'a> From<&'a str>,
 {
     // NOTE: We have a disadvantage to `String` because we cannot call `estimated_capacity()` on args
     // As such, we cannot assume a given needed capacity - we start with a stack allocated buffer
@@ -885,6 +968,50 @@ where
     builder_into!(builder, buffer)
 }
 
+/// Equivalent to `flex_fmt` except that it uses `ufmt` which is much faster, but has limitations.
+/// See [ufmt docs](https://docs.rs/ufmt/latest/ufmt/) for more details
+/// ```
+/// use flexstr::{flex_str, flex_ufmt};
+///
+/// let a = flex_ufmt!("Is {}{}", flex_str!("inlined"), "!");
+/// assert!(a.is_inlined());
+/// assert_eq!(a, "Is inlined!");
+/// ```
+#[cfg(feature = "fast_format")]
+#[macro_export(local_inner_macros)]
+macro_rules! flex_ufmt {
+    ($($arg:tt)*) => {{
+        let mut buffer = buffer_new!({ $crate::STRING_SIZED_INLINE });
+        let mut builder = builder_new!(buffer);
+
+        ufmt::uwrite!(&mut builder, $($arg)*).expect("a formatting trait implementation returned an error");
+        let s: $crate::FlexStr = builder_into!(builder, buffer);
+        s
+    }}
+}
+
+/// Equivalent to `a_flex_fmt` except that it uses `ufmt` which is much faster, but has limitations.
+/// See [ufmt docs](https://docs.rs/ufmt/latest/ufmt/) for more details
+/// ```
+/// use flexstr::{a_flex_str, a_flex_ufmt};
+///
+/// let a = a_flex_ufmt!("Is {}{}", a_flex_str!("inlined"), "!");
+/// assert!(a.is_inlined());
+/// assert_eq!(a, "Is inlined!");
+/// ```
+#[cfg(feature = "fast_format")]
+#[macro_export(local_inner_macros)]
+macro_rules! a_flex_ufmt {
+    ($($arg:tt)*) => {{
+        let mut buffer = buffer_new!({ $crate::STRING_SIZED_INLINE });
+        let mut builder = builder_new!(buffer);
+
+        ufmt::uwrite!(&mut builder, $($arg)*).expect("a formatting trait implementation returned an error");
+        let s: $crate::AFlexStr = builder_into!(builder, buffer);
+        s
+    }}
+}
+
 /// `FlexStr` equivalent to `format!` macro from stdlib. Efficiently creates a native `FlexStr`
 /// ```
 /// use flexstr::flex_fmt;
@@ -940,7 +1067,7 @@ mod tests {
 
         // Create our struct and values and verify storage
         let test = Test {
-            a: a.into(),
+            a: flex_str!(a),
             b: b.to_string().into(),
             c: c.to_string().into(),
         };
