@@ -1,39 +1,39 @@
 use alloc::string::String;
 use core::ops::Deref;
 
-use crate::{AFlexStr, FlexStr, FlexStrInner, STRING_SIZED_INLINE};
+use crate::{AFlexStr, Flex, FlexInner, FlexStr, STRING_SIZED_INLINE};
 
 // *** Repeat custom trait ***
 
 /// Trait that can repeat a given `FlexStr` "n" times efficiently
 pub trait Repeat<const N: usize, T> {
     /// Repeats a given `FlexStr` "n" times efficiently and returns a new `FlexStr`
-    fn repeat_n(&self, n: usize) -> FlexStr<N, T>;
+    fn repeat_n(&self, n: usize) -> Flex<N, T>;
 }
 
-impl<const N: usize, T> Repeat<N, T> for FlexStr<N, T>
+impl<const N: usize, T> Repeat<N, T> for Flex<N, T>
 where
-    T: Deref<Target = str> + From<String> + for<'a> From<&'a str>,
+    T: Deref<Target = str> + for<'a> From<&'a str>,
 {
     /// ```
-    /// use flexstr::{IntoFlexStr, Repeat};
+    /// use flexstr::{flex_str, IntoFlexStr, Repeat};
     ///
-    /// let s = "a".into_flex_str().repeat_n(10);
+    /// let s = flex_str!("a").repeat_n(10);
     /// assert!(s.is_inlined());
     /// assert_eq!(s, "a".repeat(10));
     /// ```
     #[inline]
-    fn repeat_n(&self, n: usize) -> FlexStr<N, T> {
+    fn repeat_n(&self, n: usize) -> Flex<N, T> {
         str::repeat_n(self, n)
     }
 }
 
 impl<const N: usize, T> Repeat<N, T> for str
 where
-    T: From<String> + for<'a> From<&'a str>,
+    T: for<'a> From<&'a str>,
 {
     #[inline]
-    fn repeat_n(&self, n: usize) -> FlexStr<N, T> {
+    fn repeat_n(&self, n: usize) -> Flex<N, T> {
         let cap = self.len() * n;
         let mut buffer = buffer_new!(N);
         let mut builder = builder_new!(buffer, cap);
@@ -51,70 +51,70 @@ where
 /// Trait that provides uppercase/lowercase conversion functions for `FlexStr`
 pub trait ToCase<const N: usize, T> {
     /// Converts string to uppercase and returns a `FlexStr`
-    fn to_upper(&self) -> FlexStr<N, T>;
+    fn to_upper(&self) -> Flex<N, T>;
 
     /// Converts string to lowercase and returns a `FlexStr`
-    fn to_lower(&self) -> FlexStr<N, T>;
+    fn to_lower(&self) -> Flex<N, T>;
 
     /// Converts string to ASCII uppercase and returns a `FlexStr`
-    fn to_ascii_upper(&self) -> FlexStr<N, T>;
+    fn to_ascii_upper(&self) -> Flex<N, T>;
 
     /// Converts string to ASCII lowercase and returns a `FlexStr`
-    fn to_ascii_lower(&self) -> FlexStr<N, T>;
+    fn to_ascii_lower(&self) -> Flex<N, T>;
 }
 
-impl<const N: usize, T> ToCase<N, T> for FlexStr<N, T>
+impl<const N: usize, T> ToCase<N, T> for Flex<N, T>
 where
-    T: Deref<Target = str> + From<String> + for<'a> From<&'a str>,
+    T: Deref<Target = str> + for<'a> From<&'a str>,
 {
     /// ```
-    /// use flexstr::{FlexStr, IntoFlexStr, ToCase};
+    /// use flexstr::{flex_str, FlexStr, ToCase};
     ///
-    /// let a: FlexStr = "test".into_flex_str().to_upper();
+    /// let a: FlexStr = flex_str!("test").to_upper();
     /// assert_eq!(a, "TEST");
     /// ```
     #[inline]
-    fn to_upper(&self) -> FlexStr<N, T> {
+    fn to_upper(&self) -> Flex<N, T> {
         str::to_upper(self)
     }
 
     /// ```
-    /// use flexstr::{FlexStr, IntoFlexStr, ToCase};
+    /// use flexstr::{flex_str, FlexStr, IntoFlexStr, ToCase};
     ///
-    /// let a: FlexStr = "TEST".into_flex_str().to_lower();
+    /// let a: FlexStr = flex_str!("TEST").to_lower();
     /// assert_eq!(a, "test");
     /// ```
     #[inline]
-    fn to_lower(&self) -> FlexStr<N, T> {
+    fn to_lower(&self) -> Flex<N, T> {
         str::to_lower(self)
     }
 
     /// ```
-    /// use flexstr::{FlexStr, IntoFlexStr, ToCase};
+    /// use flexstr::{flex_str, FlexStr, IntoFlexStr, ToCase};
     ///
-    /// let a: FlexStr = "test".into_flex_str().to_ascii_upper();
+    /// let a: FlexStr = flex_str!("test").to_ascii_upper();
     /// assert_eq!(a, "TEST");
     /// ```
     #[inline]
-    fn to_ascii_upper(&self) -> FlexStr<N, T> {
+    fn to_ascii_upper(&self) -> Flex<N, T> {
         str::to_ascii_upper(self)
     }
 
     /// ```
-    /// use flexstr::{FlexStr, IntoFlexStr, ToCase};
+    /// use flexstr::{flex_str, FlexStr, IntoFlexStr, ToCase};
     ///
-    /// let a: FlexStr = "TEST".into_flex_str().to_ascii_lower();
+    /// let a: FlexStr = flex_str!("TEST").to_ascii_lower();
     /// assert_eq!(a, "test");
     /// ```
     #[inline]
-    fn to_ascii_lower(&self) -> FlexStr<N, T> {
+    fn to_ascii_lower(&self) -> Flex<N, T> {
         str::to_ascii_lower(self)
     }
 }
 
 impl<const N: usize, T> ToCase<N, T> for str
 where
-    T: From<String> + for<'a> From<&'a str>,
+    T: for<'a> From<&'a str>,
 {
     /// ```
     /// use flexstr::{FlexStr, ToCase};
@@ -122,7 +122,7 @@ where
     /// let a: FlexStr = "test".to_upper();
     /// assert_eq!(a, "TEST");
     /// ```
-    fn to_upper(&self) -> FlexStr<N, T> {
+    fn to_upper(&self) -> Flex<N, T> {
         // We estimate capacity based on previous string, but if not ASCII this might be wrong
         let mut buffer = buffer_new!(N);
         let mut builder = builder_new!(buffer, self.len());
@@ -143,7 +143,7 @@ where
     /// let a: FlexStr = "TEST".to_lower();
     /// assert_eq!(a, "test");
     /// ```
-    fn to_lower(&self) -> FlexStr<N, T> {
+    fn to_lower(&self) -> Flex<N, T> {
         // We estimate capacity based on previous string, but if not ASCII this might be wrong
         let mut buffer = buffer_new!(N);
         let mut builder = builder_new!(buffer, self.len());
@@ -164,7 +164,7 @@ where
     /// let a: FlexStr = "test".to_ascii_upper();
     /// assert_eq!(a, "TEST");
     /// ```
-    fn to_ascii_upper(&self) -> FlexStr<N, T> {
+    fn to_ascii_upper(&self) -> Flex<N, T> {
         let mut buffer = buffer_new!(N);
         let mut builder = builder_new!(buffer, self.len());
 
@@ -182,7 +182,7 @@ where
     /// let a: FlexStr = "TEST".to_ascii_lower();
     /// assert_eq!(a, "test");
     /// ```
-    fn to_ascii_lower(&self) -> FlexStr<N, T> {
+    fn to_ascii_lower(&self) -> Flex<N, T> {
         let mut buffer = buffer_new!(N);
         let mut builder = builder_new!(buffer, self.len());
 
@@ -197,7 +197,7 @@ where
 
 // *** Generic `To` trait ***
 
-/// A trait that converts the source to a `FlexStr<N, T>` without consuming it
+/// A trait that converts the source to a `Flex<N, T>` without consuming it
 /// ```
 /// use flexstr::{FlexStr, ToFlex};
 ///
@@ -205,11 +205,11 @@ where
 /// assert!(a.is_heap());
 /// ```
 pub trait ToFlex<const N: usize, T> {
-    /// Converts the source to a `FlexStr<N, T>` without consuming it
-    fn to_flex(&self) -> FlexStr<N, T>;
+    /// Converts the source to a `Flex<N, T>` without consuming it
+    fn to_flex(&self) -> Flex<N, T>;
 }
 
-impl<const N: usize, T, T2> ToFlex<N, T> for FlexStr<N, T2>
+impl<const N: usize, T, T2> ToFlex<N, T> for Flex<N, T2>
 where
     T: for<'a> From<&'a str>,
     T2: Clone + Deref<Target = str>,
@@ -222,7 +222,7 @@ where
     /// assert_eq!(a, b);
     /// ```
     #[inline]
-    fn to_flex(&self) -> FlexStr<N, T> {
+    fn to_flex(&self) -> Flex<N, T> {
         self.clone().into_flex()
     }
 }
@@ -242,17 +242,14 @@ where
     /// assert!(b.is_heap())
     /// ```
     #[inline]
-    fn to_flex(&self) -> FlexStr<N, T> {
-        FlexStr(match self.try_into() {
-            Ok(s) => FlexStrInner::Inlined(s),
-            Err(_) => FlexStrInner::Heap(self.into()),
-        })
+    fn to_flex(&self) -> Flex<N, T> {
+        self.into()
     }
 }
 
 impl<const N: usize, T> ToFlex<N, T> for bool
 where
-    T: Deref<Target = str>,
+    T: for<'a> From<&'a str>,
 {
     /// ```
     /// use flexstr::{FlexStr, ToFlex};
@@ -262,14 +259,14 @@ where
     /// assert_eq!(s, "false");
     /// ```
     #[inline]
-    fn to_flex(&self) -> FlexStr<N, T> {
-        if *self { "true" } else { "false" }.into()
+    fn to_flex(&self) -> Flex<N, T> {
+        Flex::from_static(if *self { "true" } else { "false" })
     }
 }
 
 impl<const N: usize, T> ToFlex<N, T> for char
 where
-    T: for<'a> From<&'a str> + From<String> + Deref<Target = str>,
+    T: for<'a> From<&'a str> + Deref<Target = str>,
 {
     /// ```
     /// use flexstr::{FlexStr, ToFlex};
@@ -279,11 +276,12 @@ where
     /// assert_eq!(s, "☺");
     /// ```
     #[inline]
-    fn to_flex(&self) -> FlexStr<N, T> {
+    fn to_flex(&self) -> Flex<N, T> {
         (*self).into()
     }
 }
 
+#[cfg(feature = "int_convert")]
 macro_rules! impl_int_flex {
     ($($type:ty),+) => {
         $(impl<const N: usize, T> ToFlex<N, T> for $type
@@ -298,7 +296,7 @@ macro_rules! impl_int_flex {
             /// assert_eq!(s, "123");
             /// ```
             #[inline]
-            fn to_flex(&self) -> FlexStr<N, T> {
+            fn to_flex(&self) -> Flex<N, T> {
                 let mut buffer = itoa::Buffer::new();
                 buffer.format(*self).to_flex()
             }
@@ -306,8 +304,10 @@ macro_rules! impl_int_flex {
     };
 }
 
+#[cfg(feature = "int_convert")]
 impl_int_flex!(i8, u8, i16, u16, i32, u32, i64, u64, i128, u128, isize, usize);
 
+#[cfg(feature = "fp_convert")]
 macro_rules! impl_float_flex {
     ($($type:ty),+) => {
         $(impl<const N: usize, T> ToFlex<N, T> for $type
@@ -322,7 +322,7 @@ macro_rules! impl_float_flex {
             /// assert_eq!(s, "123.456");
             /// ```
             #[inline]
-            fn to_flex(&self) -> FlexStr<N, T> {
+            fn to_flex(&self) -> Flex<N, T> {
                 let mut buffer = ryu::Buffer::new();
                 buffer.format(*self).to_flex()
             }
@@ -330,31 +330,32 @@ macro_rules! impl_float_flex {
     };
 }
 
+#[cfg(feature = "fp_convert")]
 impl_float_flex!(f32, f64);
 
 // *** Generic `Into` Custom Traits ***
 
 /// A trait that converts the source to a `FlexStr<N, T>` while consuming the original
 /// ```
-/// use flexstr::{FlexStr, IntoFlex};
+/// use flexstr::{flex_str, FlexStr, IntoFlex};
 ///
-/// let a: FlexStr = "This is a wrapped static string literal no matter how long it is!!!!!".into_flex();
+/// let a: FlexStr = flex_str!("This is a wrapped static string literal no matter how long it is!!!!!");
 /// assert!(a.is_static());
 /// ```
 pub trait IntoFlex<const N: usize, T> {
     /// Converts the source to a `FlexStr<N, T>` while consuming the original
-    fn into_flex(self) -> FlexStr<N, T>;
+    fn into_flex(self) -> Flex<N, T>;
 }
 
-impl<const N: usize, T, T2> IntoFlex<N, T> for FlexStr<N, T2>
+impl<const N: usize, T, T2> IntoFlex<N, T> for Flex<N, T2>
 where
     T: for<'a> From<&'a str>,
     T2: Deref<Target = str>,
 {
     /// ```
-    /// use flexstr::{AFlexStr, FlexStr, IntoFlex};
+    /// use flexstr::{a_flex_str, AFlexStr, FlexStr, IntoFlex};
     ///
-    /// let a: AFlexStr = "This can be just wrapped as a static string literal!".into_flex();
+    /// const a: AFlexStr = a_flex_str!("This can be just wrapped as a static string literal!");
     /// assert!(a.is_static());
     /// let b: FlexStr = a.clone().into_flex();
     /// assert!(b.is_static());
@@ -373,42 +374,24 @@ where
     /// assert_eq!(e, f);
     /// ```
     #[inline]
-    fn into_flex(self) -> FlexStr<N, T> {
-        FlexStr(match self.0 {
-            FlexStrInner::Static(s) => FlexStrInner::Static(s),
-            FlexStrInner::Inlined(s) => FlexStrInner::Inlined(s),
-            FlexStrInner::Heap(heap) => {
+    fn into_flex(self) -> Flex<N, T> {
+        Flex(match self.0 {
+            FlexInner::Static(s) => FlexInner::Static(s),
+            FlexInner::Inlined(s) => FlexInner::Inlined(s),
+            FlexInner::Heap(heap) => {
                 // TODO: Any more efficient way to do this?
                 // Would like to use `from_raw` and `into_raw`, but need to ensure
                 // exclusive ownership for this to be safe. For `Rc` that might be possible,
                 // but `Arc` could be multi-threaded so needs to be atomic
-                FlexStrInner::Heap(T::from(&heap))
+                FlexInner::Heap(T::from(&heap))
             }
         })
     }
 }
 
-impl<const N: usize, T> IntoFlex<N, T> for &'static str
-where
-    T: Deref<Target = str>,
-{
-    /// ```
-    /// use flexstr::{AFlexStr, IntoFlex};
-    ///
-    /// let a = "This can just be wrapped as a static string literal!";
-    /// let b: AFlexStr = a.into_flex();
-    /// assert!(b.is_static());
-    /// assert_eq!(b, a);
-    /// ```
-    #[inline]
-    fn into_flex(self) -> FlexStr<N, T> {
-        self.into()
-    }
-}
-
 impl<const N: usize, T> IntoFlex<N, T> for String
 where
-    T: From<String>,
+    T: for<'a> From<&'a str>,
 {
     /// ```
     /// use flexstr::{AFlexStr, IntoFlex};
@@ -419,8 +402,8 @@ where
     /// assert_eq!(b, a);
     /// ```
     #[inline]
-    fn into_flex(self) -> FlexStr<N, T> {
-        self.into()
+    fn into_flex(self) -> Flex<N, T> {
+        <Flex<N, T> as From<&str>>::from(&self)
     }
 }
 
@@ -438,7 +421,7 @@ pub trait ToFlexStr {
     fn to_flex_str(&self) -> FlexStr;
 }
 
-impl<T> ToFlexStr for FlexStr<STRING_SIZED_INLINE, T>
+impl<T> ToFlexStr for Flex<STRING_SIZED_INLINE, T>
 where
     T: Clone + Deref<Target = str>,
 {
@@ -497,6 +480,7 @@ impl ToFlexStr for char {
     }
 }
 
+#[cfg(feature = "int_convert")]
 macro_rules! impl_int_flex_str {
     ($($type:ty),+) => {
         $(impl ToFlexStr for $type
@@ -516,8 +500,10 @@ macro_rules! impl_int_flex_str {
     };
 }
 
+#[cfg(feature = "int_convert")]
 impl_int_flex_str!(i8, u8, i16, u16, i32, u32, i64, u64, i128, u128, isize, usize);
 
+#[cfg(feature = "fp_convert")]
 macro_rules! impl_float_flex_str {
     ($($type:ty),+) => {
         $(impl ToFlexStr for $type
@@ -537,6 +523,7 @@ macro_rules! impl_float_flex_str {
     };
 }
 
+#[cfg(feature = "fp_convert")]
 impl_float_flex_str!(f32, f64);
 
 // *** AFlexStr `To` Traits ***
@@ -553,7 +540,7 @@ pub trait ToAFlexStr {
     fn to_a_flex_str(&self) -> AFlexStr;
 }
 
-impl<T> ToAFlexStr for FlexStr<STRING_SIZED_INLINE, T>
+impl<T> ToAFlexStr for Flex<STRING_SIZED_INLINE, T>
 where
     T: Clone + Deref<Target = str>,
 {
@@ -612,6 +599,7 @@ impl ToAFlexStr for char {
     }
 }
 
+#[cfg(feature = "int_convert")]
 macro_rules! impl_int_a_flex_str {
     ($($type:ty),+) => {
         $(impl ToAFlexStr for $type
@@ -631,8 +619,10 @@ macro_rules! impl_int_a_flex_str {
     };
 }
 
+#[cfg(feature = "int_convert")]
 impl_int_a_flex_str!(i8, u8, i16, u16, i32, u32, i64, u64, i128, u128, isize, usize);
 
+#[cfg(feature = "fp_convert")]
 macro_rules! impl_float_a_flex_str {
     ($($type:ty),+) => {
         $(impl ToAFlexStr for $type
@@ -652,15 +642,16 @@ macro_rules! impl_float_a_flex_str {
     };
 }
 
+#[cfg(feature = "fp_convert")]
 impl_float_a_flex_str!(f32, f64);
 
 // *** FlexStr `Into` Traits ***
 
 /// A trait that converts the source to a `FlexStr` while consuming the original
 /// ```
-/// use flexstr::IntoFlexStr;
+/// use flexstr::flex_str;
 ///
-/// let a = "This is a wrapped static string literal no matter how long it is!!!!!".into_flex_str();
+/// let a = flex_str!("This is a wrapped static string literal no matter how long it is!!!!!");
 /// assert!(a.is_static());
 /// ```
 pub trait IntoFlexStr {
@@ -668,14 +659,14 @@ pub trait IntoFlexStr {
     fn into_flex_str(self) -> FlexStr;
 }
 
-impl<T> IntoFlexStr for FlexStr<STRING_SIZED_INLINE, T>
+impl<T> IntoFlexStr for Flex<STRING_SIZED_INLINE, T>
 where
     T: Deref<Target = str>,
 {
     /// ```
-    /// use flexstr::{AFlexStr, IntoFlex, IntoFlexStr};
+    /// use flexstr::{a_flex_str, AFlexStr, IntoFlex, IntoFlexStr};
     ///
-    /// let a: AFlexStr = "This can be just wrapped as a static string literal!".into_flex();
+    /// let a: AFlexStr = a_flex_str!("This can be just wrapped as a static string literal!");
     /// assert!(a.is_static());
     /// let b = a.clone().into_flex_str();
     /// assert!(b.is_static());
@@ -687,19 +678,6 @@ where
     }
 }
 
-impl IntoFlexStr for &'static str {
-    /// ```
-    /// use flexstr::IntoFlexStr;
-    ///
-    /// let a = "This is a wrapped static string literal no matter how long it is!!!!!".into_flex_str();
-    /// assert!(a.is_static());
-    /// ```
-    #[inline]
-    fn into_flex_str(self) -> FlexStr {
-        self.into()
-    }
-}
-
 impl IntoFlexStr for String {
     /// ```
     /// use flexstr::IntoFlexStr;
@@ -709,7 +687,7 @@ impl IntoFlexStr for String {
     /// ```
     #[inline]
     fn into_flex_str(self) -> FlexStr {
-        self.into()
+        self.into_flex()
     }
 }
 
@@ -717,9 +695,9 @@ impl IntoFlexStr for String {
 
 /// A trait that converts the source to a `AFlexStr` while consuming the original
 /// ```
-/// use flexstr::IntoAFlexStr;
+/// use flexstr::a_flex_str;
 ///
-/// let a = "This is a wrapped static string literal no matter how long it is!!!!!".into_a_flex_str();
+/// let a = a_flex_str!("This is a wrapped static string literal no matter how long it is!!!!!");
 /// assert!(a.is_static());
 /// ```
 pub trait IntoAFlexStr {
@@ -727,14 +705,14 @@ pub trait IntoAFlexStr {
     fn into_a_flex_str(self) -> AFlexStr;
 }
 
-impl<T> IntoAFlexStr for FlexStr<STRING_SIZED_INLINE, T>
+impl<T> IntoAFlexStr for Flex<STRING_SIZED_INLINE, T>
 where
     T: Deref<Target = str>,
 {
     /// ```
-    /// use flexstr::{AFlexStr, IntoFlex, IntoAFlexStr};
+    /// use flexstr::{AFlexStr, IntoAFlexStr, a_flex_str};
     ///
-    /// let a: AFlexStr = "This can be just wrapped as a static string literal!".into_flex();
+    /// let a: AFlexStr = a_flex_str!("This can be just wrapped as a static string literal!");
     /// assert!(a.is_static());
     /// let b = a.clone().into_a_flex_str();
     /// assert!(b.is_static());
@@ -746,19 +724,6 @@ where
     }
 }
 
-impl IntoAFlexStr for &'static str {
-    /// ```
-    /// use flexstr::IntoAFlexStr;
-    ///
-    /// let a = "This is a wrapped static string literal no matter how long it is!!!!!".into_a_flex_str();
-    /// assert!(a.is_static());
-    /// ```
-    #[inline]
-    fn into_a_flex_str(self) -> AFlexStr {
-        self.into()
-    }
-}
-
 impl IntoAFlexStr for String {
     /// ```
     /// use flexstr::IntoAFlexStr;
@@ -768,6 +733,6 @@ impl IntoAFlexStr for String {
     /// ```
     #[inline]
     fn into_a_flex_str(self) -> AFlexStr {
-        self.into()
+        self.into_flex()
     }
 }
